-- |Das Modul enthlt eine Funktion @dynamic@ zur Lsung von Problemen
-- mittels dynamischen Programmierens.
--
-- Aus: Fethi Rabhi, Guy Lapalme, "Algorithms - a functional approach", p. 180ff.

module Gersdorf.Algorithms.Dynamic (
    DynTable,
    dynamic,
    dynamicSolve
) where

import Data.Array

newtype (Ix it) => DynTable it et = DynTable (Array it et)
    deriving Show

-- |Funktion, die aus einer DynTable eine bestimmte Lsung liest. Damit
-- wird die Auswertung der Tabelleninhalte angestoen, die Sublsungen lazy
-- berechnet.
dynamicSolve :: (Ix it) => DynTable it et -> it -> et
dynamicSolve (DynTable arr) coord = arr ! coord

-- |Funktion zur Erstellung einer Tabelle zum dynamischen Programmieren.
-- Die einzelnen Zellen werden mit einem Paar aus Koordinaten und eine Funktion,
-- die aus den Koordinaten die Lsung dieser Zelle berechnet, belegt.
--
-- Die Tabellenimplementierung beruht auf IArrays, die ein O(1)-Lookup besitzen.
-- Durch die lazy-Auswertung wird jede Zelle hchstens einmal berechnet.
dynamic :: (Ix it) => (DynTable it et -> it -> et) -> (it, it) -> DynTable it et
dynamic compute bnds = t
    where t = DynTable (array bnds (map (\coord -> (coord, compute t coord)) (range bnds)))
