module Gersdorf.ContextMatching.Base.HeapTerm (
   HeapTerm,
   HeapPath,
   toHeapTerm,
   module Gersdorf.ContextMatching.Base.PositionTerm
) where

import Data.Maybe

import Gersdorf.Data.HeapTree 
import Gersdorf.ContextMatching.Base.ExtendedTerm 
import Gersdorf.ContextMatching.Base.Term
import Gersdorf.ContextMatching.Base.Utilities
import Gersdorf.ContextMatching.Base.PositionTerm
import Gersdorf.Data.Tree
import Debug.Trace

type HeapTerm = HeapTree TermType

instance Term HeapTerm

instance Linearizable HeapTerm where
    linearize ss t = 
        let exTerm = toExtendedTerm t
            (finalstate, linearizedS) = linearize ss exTerm
        in  (finalstate, toHeapTree linearizedS)

instance TermTypeable HeapTerm where
    getType = getContent

instance TermIterator HeapTerm 

instance Show HeapTerm where
    showsPrec _ t r = showTerm t r

instance PositionTerm HeapTerm HeapPath where
    makeContextPair t = 
        let holeTermNode = extractNode $ toHeapTerm $ (cTerm $ cHole :: ExtendedTerm)
        in  (replaceSubtree t holeTermNode, extractNode t)
    makeMonadicTerm t = 
        let et = toExtendedTerm (getRoot t)
        in  toHeapTerm $ path2MonadicTreeC et (fromJust $ cPath et $ toPathIndexList $ getPath t)

instance AdressableTerm HeapTerm HeapPath

toHeapTerm :: (Term t) => 
    t
    -> HeapTerm
toHeapTerm = cTree . convertTree . toExtendedTerm

instance (Term t1) => ConvertableTerm t1 HeapTerm where
    convertTerm = toHeapTerm
