-- |Ein PositionTerm ist ein Term, dessen Subterme ber Positionen 
--  angesprochen werden knnen.
module Gersdorf.ContextMatching.Base.PositionTerm (
    PositionTerm(..),
    AdressableTerm(..),
    permuteContexts,
    permuteContextTerms,
    module Gersdorf.ContextMatching.Base.Term
) where

import Gersdorf.ContextMatching.Base.Term
import Gersdorf.Data.Tree
import Debug.Trace

class (UpdatableTree t tp, Term t) => PositionTerm t tp where

    getHolePath :: t -> Maybe tp 
    getHolePath t = case (filter isHole $ toList t) of
                     [] -> Nothing
                     (t:_) -> Just (getPath t)

    instantiateContext :: t -> TermNode t -> Maybe t
    instantiateContext ctx t = 
        do hp <- getHolePath ctx
           replacePath ctx hp t

   -- |Erstellt einen neuen PositionTerm t', indem im alten PositionTerm t
   -- an der Position p ein Loch eingesetzt wird.
    makeContext :: t     -- ^ the subterm to be replaced by a Hole
        -> t             -- ^ the root of the newly generated tree
    makeContext = fst . makeContextPair 

    makeContextPair :: t    -- ^ Der Subterm, der durch ein Loch ersetzt werden soll
        -> (t, TreeNode t)  -- ^ (Wurzel des Zielterms, in dem ein Loch eingesetzt wurde; Term, der durch Loch ersetzt wurde)

    makeContextPathPair :: (TreeExtractor t) => 
	t                -- ^ Der Zielterm. Kann auch ein Subterm sein. Ein Subterm dieses Zielterms wird durch ein Loch ersetzt.
        -> tp            -- ^ Pfad zum einzusetzenden Loch
        -> Maybe (t, TreeNode t) -- ^ (Wurzel des Zielterms, in dem ein Loch eingesetzt wurde, Term, der durch Loch ersetzt wurde)
    makeContextPathPair target holePath =
	do let targetPathDepth = pathDepth (getPath target)
           let newRoot         = cTree $ extractNode target

           newPath     <- cPath newRoot $ drop targetPathDepth $ toPathIndexList holePath
	   holeSubTerm <- gotoPath newRoot newPath

           return (makeContext holeSubTerm, extractNode holeSubTerm)

    makeContextPath :: (TreeExtractor t) => 
	t                -- ^ Der Zielterm. Kann auch ein Subterm sein. Ein Subterm dieses Zielterms wird durch ein Loch ersetzt.
        -> tp            -- ^ Pfad zum einzusetzenden Loch
        -> Maybe t       -- ^ Wurzel des Zielterms, in dem ein Loch eingesetzt wurde.
    makeContextPath target holePath =
	do (context, _) <- makeContextPathPair target holePath
           return context

    makeMonadicTerm :: t -> t

class (PositionTerm t tp, Adressable t, AdressTarget t ) => AdressableTerm t tp where
    makeContextAdress :: (TreeExtractor t) =>
	t                -- ^ Der Zielterm. Kann auch ein Subterm sein. Ein Subterm dieses Zielterms wird durch ein Loch ersetzt.	
	-> Int           -- ^ Adresse zum Subterm, der durch ein Loch ersetzt werden soll
	-> Maybe t       -- ^ Wurzel des Zielterms, in dem ein Loch eingesetzt wurde.
    makeContextAdress target holeAdress =
	let holePath        = getPath (getAt target holeAdress)
	in  makeContextPath target holePath

permuteContexts :: (PositionTerm t tp) =>
	t
	-> [(t, TreeNode t)]
permuteContexts t = map makeContextPair (toList t)

permuteContextTerms :: (PositionTerm t tp, TreeExtractor t) =>
	t
	-> [(t, t)]
permuteContextTerms t = map (\(t, tn) -> (t, cTree tn)) (permuteContexts t)
