module Gersdorf.ContextMatching.Base.SemiSolvedSubstitution (
   SemiSolvedSubstitution,
   module Gersdorf.ContextMatching.Base.Substitution
) where

import Gersdorf.ContextMatching.Base.Utilities
import Gersdorf.ContextMatching.Base.HeapTerm
import Gersdorf.ContextMatching.Base.Substitution
import Data.Maybe
import Debug.Trace

data (PositionTerm t tp) => SemiSolvedSubstitution t tp = 
    SemiBot | SemiTop
    | SemiVar TermType t
    | SemiContextVar TermType t 
    | SemiAnd (SemiSolvedSubstitution t tp) (SemiSolvedSubstitution t tp)
    | SemiOr (SemiSolvedSubstitution t tp) (SemiSolvedSubstitution t tp)
    deriving Eq

instance (Show t, PositionTerm t tp) => Show (SemiSolvedSubstitution t tp) where
    showsPrec _ SemiBot r = showString "<bot>" r
    showsPrec _ SemiTop r = showString "<top>" r
    showsPrec _ (SemiVar var t) r = showString "<" (shows t (showString "/" (shows var (showString ">" r))))
    showsPrec _ (SemiContextVar cvar t) r = showString "<" (shows t (showString ("/" ++ (getName cvar) ++ ">") r))
    showsPrec _ (SemiAnd s1 s2) r = showPredList "(" [s1,s2] " AND " ")" r
    showsPrec _ (SemiOr s1 s2) r = showPredList "(" [s1,s2] " OR " ")" r

instance (PositionTerm t tp) => BotToppable (SemiSolvedSubstitution t tp) where
    isTop SemiTop = True
    -- Ein Und bzw. Oder trgt nur dann nichts zur Lsung bei, wenn es komplett Top ist.
    isTop (SemiAnd s1 s2) = (isTop s1 && isTop s2)
    isTop (SemiOr s1 s2) = (isTop s1 && isTop s2)
    isTop _ = False
 
    isBot SemiBot = True
    isBot (SemiAnd s1 s2) = (isBot s1 || isBot s2)
    isBot (SemiOr s1 s2) = (isBot s1 || isBot s2)
    isBot _ = False

    isFlatTop SemiTop = True
    isFlatTop _ = False
    isFlatBot SemiBot = True
    isFlatBot _ = False

instance SubstitutionBuilder (SemiSolvedSubstitution HeapTerm HeapPath) HeapTerm HeapPath where
    substTop = SemiTop
    substBot = SemiBot

    substVar var target = SemiVar (getType var) target
    substContextVar cvar target holePos =
        SemiContextVar (getType cvar) (fromJust $ makeContextAdress target holePos)
	    
    substAnd [] = substTop
    substAnd [s] = s
    substAnd (s:ss) = substAnd2 s (substAnd ss)
        where substAnd2 s1 s2 = if (isFlatBot s1 || isFlatBot s2) then SemiBot else SemiAnd s1 s2

    substOr [] = substTop
    substOr [s] = s
    substOr (s:ss) = substOr2 s (substOr ss)
        where substOr2 s1 s2 = if (isFlatBot s1 && isFlatBot s2) then SemiBot else SemiOr s1 s2

instance SubstitutionGenerator (SemiSolvedSubstitution HeapTerm HeapPath) HeapTerm where
    generate s = filter (not . isBotTop) result
        where
        result = map (\(Substitution ss) -> Substitution $ filter (not . isTop) ss) (generate' s)
 
        -- Top darf nicht gefiltert werden ,da es noch fr die Listcomprehension bentigt wird.
        -- wird in der Toplevel-Funktion gefiltert!
        generate' s = filter (not . isBot) (generate'' s)

        -- Wir mssen hier SubstTop generieren wegen der List Comprehension bei SemiAnd.
        -- sonst ist eine der Listen leer und leere Listen erzeugen keine Elemente.
        generate'' SemiTop = [Substitution [SubstTop]]
        generate'' SemiBot = [Substitution [SubstBot]]
        generate'' (SemiVar v t) = [Substitution [SubstVar v t]]
        generate'' (SemiContextVar v t) = [Substitution [SubstContextVar v t]]

        generate'' (SemiAnd s1 s2) = [ Substitution (css1 ++ css2) 
                                        | (Substitution css1) <- (generate' s1), (Substitution css2) <- (generate' s2) ]

        generate'' (SemiOr s1 s2) = (generate' s1) ++ (generate' s2)

{-    simplify (SemiOr s1 s2) = 
        case dec of
            (True, True, _, _) -> SemiBot
            (_, _, True, True) -> SemiTop
            (_, _, True, False) -> ss2
            (_, _, False, True) -> ss1
            _ -> SemiOr ss1 ss2
        where ss1 = simplify s1
              ss2 = simplify s2
              dec@(fbot1, fbot2, ftop1, ftop2) = (isFlatBot ss1, isFlatBot ss2, isFlatTop ss1, isFlatTop ss2)

    simplify (SemiAnd s1 s2) =
        case dec of
            (True, True, _, _) -> SemiBot
            (True, False, _, _) -> SemiBot
            (False, True, _, _) -> SemiBot
            (_, _, True, True) -> SemiTop
            (_, _, True, False) -> ss2
            (_, _, False, True) -> ss1
            _ -> SemiOr ss1 ss2
        where ss1 = simplify s1
              ss2 = simplify s2
              dec@(fbot1, fbot2, ftop1, ftop2) = (isFlatBot ss1, isFlatBot ss2, isFlatTop ss1, isFlatTop ss2)
-}
    simplify s = s
