
-- |This is the module containing a very simple implementation
-- of Terms.

module Gersdorf.ContextMatching.Base.SimpleTerm (
    toSimpleTerm,
    SimpleTerm,
    SimpleTermNode,
    StandardTreePath,
    module Gersdorf.ContextMatching.Base.Term
) where

import Gersdorf.Data.ExtendedTree
import Gersdorf.ContextMatching.Base.Term 
import Gersdorf.ContextMatching.Base.PositionTerm
import Debug.Trace

type SimpleTerm = SimpleTermT TermType
type SimpleTermNode = SimpleTermNodeT TermType

type SimpleTermT tt = ExtendedTree tt
type SimpleTermNodeT tt = TermNode (SimpleTermT tt)

instance (TermTypeable tt) => TermTypeable (SimpleTermT tt) where
    -- Get Name of the root of this term
    getName t = getName (getType t)
    getType = getType . getContent

instance (TermTypeBuilder tt) => Linearizable (SimpleTermT tt) where
    linearize ss t = 
        let transformStateF c t
                | isVariable t         = (c+1, TAThisNode (\_ -> cVar (newname t c)))
                | isContextVariable t  = (c+1, TAThisNode (\css -> cContextVar (newname t c) (head css)))
                | otherwise            = (c, TANothing)
            newname t c = (getName t) ++ (show c)
        in  transformTreeState transformStateF ss t

instance (TermTypeBuilder tt) =>Term (SimpleTermT tt)

instance (TermTypeBuilder tt) => TermIterator (SimpleTermT tt)

instance (TermTypeBuilder tt) => Show (SimpleTermT tt) where
    showsPrec _ t r = showTerm t r

instance (TermTypeBuilder tt) => SubTermBuilder (SimpleTermT tt) where
    cVar n = cNode (cVarType n) [] 
    cFunction n tss = cNode (cFunctionType n) tss
    cContextVar n t = cNode (cContextVarType n) [t]
    cHole = cNode cHoleType []

instance (TermTypeBuilder tt) => TermBuilder (SimpleTermT tt) where
    -- Implementation of factory members
    cFunctionT n tss = cFunction n (map extractNode tss)
    cContextVarT n t = cContextVar n (extractNode t)

    cTerm = cTree

    replaceTerm ct ntn = replaceSubtree ct ntn

    extractSubTerm = cTree . extractNode

{-toSimpleTerm :: (ConvertableContainerTree t ExtendedTree TermType, Term (t TermType)) =>
    t TermType
    -> SimpleTermT
toSimpleTerm = cTree . convertTree 
-}
{--instance (ConvertableContainerTree t1 t2 tt, TermTypeable tt) => ConvertableTerm (t1 TermType) (ExtendedTree TermType) where
    convertTerm t1 = toSimpleTerm t1 --cTree $ (convertTree t1) 
--}

toSimpleTerm :: (Term t) => t -> SimpleTerm
toSimpleTerm t = cTree $ toSimpleTerm' t
	where
	toSimpleTerm' t | isHole t     = cHole
			| isVariable t = cVar (getName t)
			| isContextVariable t = cContextVar (getName t) (toSimpleTerm' $ head $ getChildren t)
			| isFunction t = cFunction (getName t) (map toSimpleTerm' (getChildren t))

instance (TermTypeBuilder tt) => PositionTerm (SimpleTermT tt) StandardTreePath where
    makeContextPair t = (replaceSubtree t cHole, extractNode t)
    makeMonadicTerm t = path2MonadicTree (getRoot t) (getPath t)
	
instance  (Term t1) => ConvertableTerm t1 SimpleTerm where
    convertTerm = toSimpleTerm
