-- |Das Modul enthlt eine einfache Implementierung fr
-- Termgleichungen
module Gersdorf.ContextMatching.Base.SimpleTermEquation (
    SimpleTermEquation
) where

import Gersdorf.ContextMatching.Base.TermEquation
import Gersdorf.ContextMatching.Base.Term hiding (size)
import qualified Gersdorf.ContextMatching.Base.Term (size)
import Gersdorf.ContextMatching.Base.Substitution

-- have to use data declaration here because of the type class
-- restriction
data (Term t) => SimpleTermEquation t = SimpleTermEquation t t
    deriving Eq

instance (Term t) => Show (SimpleTermEquation t) where
    showsPrec _ (SimpleTermEquation t1 t2) r = shows t1 (showString " =? " (shows t2 r))

instance (Term t) => TermEquation SimpleTermEquation t where
    cEquation t1 t2 = SimpleTermEquation t1 t2

    sizeEq (SimpleTermEquation t1 t2) = max (Gersdorf.ContextMatching.Base.Term.size t1)
                                          (Gersdorf.ContextMatching.Base.Term.size t2)

    getLeft (SimpleTermEquation t1 _) = t1
    getRight (SimpleTermEquation _  t2) = t2

instance (Term t, Substitutable t) => Substitutable (SimpleTermEquation t) where
    applySubstitution te sub = 
        let l = getLeft te
            substituted = snd $ applySubstitution l sub
        in  (sub, cEquation substituted (getRight te))

    isCorrectUnderSubst te sub = isCorrectEq $ snd $ applySubstitution te sub

instance (Term t) => Linearizable (SimpleTermEquation t) where
    linearize s te = 
        let (leftState, leftLin) = linearize s (getLeft te)
            (rightState, rightLin) = linearize leftState (getRight te)
        in  (rightState, cEquation leftLin rightLin)
