module Gersdorf.ContextMatching.Base.SimpleUnificationProblem (
    SimpleUnificationProblem,
    UnificationProblem(..),
    module Gersdorf.ContextMatching.Base.ListEquationSet
) where

import Gersdorf.ContextMatching.Base.UnificationProblem
import Gersdorf.ContextMatching.Base.ExtendedTerm (cFunction, toExtendedTerm)
import Gersdorf.ContextMatching.Base.ListEquationSet
import Gersdorf.ContextMatching.Base.Substitution
import Gersdorf.ContextMatching.Base.SimpleTermEquation

type SimpleUnificationProblem e t = ListEquationSet e t

instance (TermEquation e t) => UnificationProblem ListEquationSet e t where
   cProblem = cSet

   isMatchingP tes = all isMatchingEq (toEquationList tes)

   -- TODO: das ist eine primitive Implementierung; hier muss noch
   -- TODO: das Erzeugen des neuen Funktionsnamens implementiert werden!
   -- Ergibt einen Simpleterm!
   toSingleTermEquation tes =
                cEquation (cTerm $ cFunctionT newFunctionName lefts) (cTerm $ cFunctionT newFunctionName rights)
       where newFunctionName = "#_$NewFunction123$_#"
             lefts = extract getLeft
             rights = extract getRight
             extract f = map (\e -> toExtendedTerm $ f e) (toEquationList tes)


instance (Term t, Substitutable t) =>
    Substitutable (ListEquationSet SimpleTermEquation t) where
    
    applySubstitution up sub = 
        let eqs = toEquationList up
            substituted = map (\eq -> snd (applySubstitution eq sub)) eqs
        in  (sub, cSet substituted)

    isCorrectUnderSubst up sub = isCorrectUP $ snd $ applySubstitution up sub

instance (TermEquation e t) => Linearizable (ListEquationSet e t) where
    linearize ss up = 
        let (finalState, finalEqs) = linearize' ss [] (toEquationList up)
            linearize' curState accum []     = (curState, accum)
            linearize' curState accum (e:es) = 
                let (nextState, linEq) = linearize curState e
                in  linearize' nextState (linEq:accum) es
        in  (finalState, cProblem finalEqs)
