-- |Das Modul enthlt eine Typklasse fr Term-Gleichungen
module Gersdorf.ContextMatching.Base.TermEquation (
    TermEquation(..),
    childrenExtract,
    module Gersdorf.ContextMatching.Base.Term
) where

import Gersdorf.Data.List

import Gersdorf.ContextMatching.Base.Term

-- |Die Typklasse @TermEquation@ umfasst alle Typen, die Termgleichungen
-- darstellen.
class (Linearizable (te t), Show (te t), Term t, Eq (te t)) => TermEquation te t where
    -- |factory method to create a termequation
    cEquation :: t -> t -> (te t)

    -- |Return size of equation
    sizeEq :: te t -> Int

    -- |Return left hand side Term
    getLeft :: te t -> t
    -- |Return right hand side Term
    getRight :: te t -> t
    -- |Return the Equation as a Pair
    getPair :: te t -> (t, t)
    getPair eq = (getLeft eq, getRight eq)

    -- |Test if this Equation is a Matching equation
    isMatchingEq :: te t -> Bool
    isMatchingEq = isGround . getRight

    -- |Test, ob diese Gleichung gelst ist
    isSolved :: te t -> Bool
    isSolved eq = (isVariable left) || 
                  (isContextVariable left && isHole (head $ getChildren left))
        where
        left = getLeft eq

    -- |Test, ob linke und rechte Seite syntaktisch gleich sind.
    isCorrectEq :: te t -> Bool
    isCorrectEq te = (getLeft te == getRight te)

    -- | Holt aus der Termgleichung Paare von Lchern, die an der gleichen Stelle stehen.
    getMatchingHoles :: te t -> [(t, t)]
    getMatchingHoles eq 
        | countHoles l /= countHoles r = []
        | otherwise = zip (getHolesDeep l) (getHolesDeep r)
        where
        l = getLeft eq
        r = getRight eq

    getMatchingConstants :: te t -> [(t, t)]
    getMatchingConstants eq = zipEquals lc rc
        where
        lc = getUniqueConstants $ getLeft eq
        rc = getUniqueConstants $ getRight eq

-- |Baut aus den n Kindern von zwei Termen n Termgleichungen auf.
childrenExtract :: (Term t, TreeExtractor t, TermEquation te t) =>
    t      -- ^ erster Term,
    -> t   -- ^ zweiter Term
    -> [te t]
childrenExtract l r =
    map childrenExtract' $ zip (getChildren l) (getChildren r)
    where
    childrenExtract' (t1, t2) =
        let newt1 = cTree $ extractNode t1
            newt2 = cTree $ extractNode t2
        in  (cEquation $! newt1) $! newt2            
