-- |Ein Unifikationsproblem ist eine Menge von Termgleichungen.
-- Abhngig davon, ob alle rechten Seiten "ground" sind, ist 
-- das Unifikationsproblem tatschlich ein Matchingproblem.

module Gersdorf.ContextMatching.Base.UnificationProblem (
    UnificationProblem(..),
    module Gersdorf.ContextMatching.Base.EquationSet
) where

import Gersdorf.ContextMatching.Base.EquationSet
import Gersdorf.ContextMatching.Base.ExtendedTerm 
import Gersdorf.ContextMatching.Base.SimpleTermEquation
import Gersdorf.ContextMatching.Base.Term
import Gersdorf.ContextMatching.Base.Substitution

class (Linearizable (up e t), Show (up e t), EquationSet up e t) => UnificationProblem up e t where
    -- |factory method to create a new UnificationProblem
    cProblem :: [e t] -> (up e t)

    -- |check if unification problem is in fact a matching problem
    isMatchingP :: up e t -> Bool

    -- |convert the UnificationProblem into one single Termequation.
    -- This is done by introducing a new n-ary function symbol
    -- where n is the number of term equations in this Unification Problem.
    toSingleTermEquation :: (Term t, TermBuilder t) => up e t -> (SimpleTermEquation ExtendedTerm)

    isCorrectUP :: up e t -> Bool
    isCorrectUP = all isCorrectEq . toEquationList
