module Gersdorf.ContextMatching.Base.Utilities (
    showPredList,
    showRecordHelper
) where

-- |Zeigt eine Liste von Werten an. Dabei werden die einzelnen Werte durch ein
-- Prdikat (einen String) getrennt.
--
-- Beispiel: @showPredList "{ " [1,2,3] " AND " " }" "REST"@ fhrt zu
--           @"{ 1 AND 2 AND 3 }REST"@.
showPredList :: (Show a) =>
    String              -- ^ Text, der vor der Liste erscheinen soll, z. B. "{"
    -> [a]              -- ^ Liste
    -> String           -- ^ Prdikat zwischen den Listenelementen  
    -> String           -- ^ Text, der nach der Liste erscheinen soll
    -> ShowS
showPredList bef list pred aft r = showString bef (showPredList' list pred (showString aft r))
    where showPredList' :: (Show a) => [a] -> String -> ShowS
          showPredList' [] _ r  = r
          showPredList' [s] _ r = shows s r
          showPredList' (s:ss) pred r = shows s (showString pred (showPredList' ss pred r))

showRecordHelper ::  
    String                 -- ^ Name of the record
    -> [(String, r -> String)]  -- ^ (fieldName, Record selectors)
    -> r                   -- ^ the record
    -> ShowS
showRecordHelper rName sels rec r =
    let outputList = map (\(fn, f) -> (fn ++ " = " ++ f rec)) sels
    in  showPredList (rName ++ " {\n") outputList "\n" "}\n" r
