module Gersdorf.ContextMatching.General.CorrespondingPositions.CorrespondingPosHoles (
    corrPosHolesProvider,
    correspondingPosHoles
) where

import Data.Maybe
import Debug.Trace

import Gersdorf.Data.Tuple

import Gersdorf.ContextMatching.Base.TermEquation
import Gersdorf.ContextMatching.Base.PositionTerm
import Gersdorf.ContextMatching.Base.HeapTerm
import Gersdorf.ContextMatching.Base.SimpleTermEquation
import Gersdorf.ContextMatching.Base.Terms
import Gersdorf.ContextMatching.Base.TermParser

import Gersdorf.ContextMatching.General.CorrespondingPositions.CorrespondingPosLinear
import Gersdorf.ContextMatching.General.Rules.Split

corrPosHolesProvider :: (PositionTerm t tp) =>
  CorrPosProvider t tp
corrPosHolesProvider =
  cCorrPosProvider "correspondingPosHoles" correspondingPosHoles True

-- | Berechnet Korrespondierende Positionen anhand von ausgehenden korrespondierenden
--   Positionen - hier von Lchern. Parallel liegende Lcher in Gleichungen markieren
--   korrespondierende Positionen. 
correspondingPosHoles :: (PositionTerm t tp) => 
    (t,t) -> [(tp, tp)]
correspondingPosHoles p =
    let mHoles             = matchingHoles p
        corrMonadics       = correspondingMonadics mHoles
        corrPairs          = zip mHoles (correspondingPairs corrMonadics)
        result = concat $ map calcCorrespondingPaths corrPairs
    in  result
--    in  trace ("*** CorrPosHoles: " ++ show corrMonadics) result

matchingHoles :: (Term t) =>
    (t, t)
    -> [(t, t)]
matchingHoles p =
    let eq :: (Term t) => (t, t) -> SimpleTermEquation t
        eq (l, r) = cEquation l r
    in  getMatchingHoles $ eq p

correspondingPairs :: (PositionTerm t tp) =>
    [(t, t)]
    -> [[(HeapPath, HeapPath)]]
correspondingPairs = map correspondingPosLinear

correspondingMonadics :: (PositionTerm t tp) =>
    [(t, t)]
    -> [(t, t)]
correspondingMonadics = map (scalarApply makeMonadicTerm)

calcCorrespondingPaths :: (PositionTerm t tp) => 
    ((t,t), [(HeapPath, HeapPath)])
    -> [(tp, tp)]
calcCorrespondingPaths ((l, r), pths) =
    let (lp, rp) = unzip pths
    in  zip (findCorrespondingPaths l lp) (findCorrespondingPaths r rp)

findCorrespondingPaths :: (PositionTerm t tp) =>
    t
    -> [HeapPath]
    -> [tp]
findCorrespondingPaths t pths =
    let r         = getRoot t
        pilist    = toPathIndexList (getPath t)
        depths    = map pathDepth pths
        origPaths = map (\i -> fromJust $ cPath r $ take i pilist) depths
    in  origPaths

corrPosEq :: ContextTermEq
corrPosEq = read "C[f(g(x), h(_))] =? r(f(g(a()), h(_)))"

eq = getPair corrPosEq
