module Gersdorf.ContextMatching.General.CorrespondingPositions.CorrespondingPosLinear (
    correspondingPosLinear,
    correspondingTermsLinear,
    corrPosLinearProvider
) where

import Data.Maybe
import Data.List

import Gersdorf.Algorithms.Dynamic

import Gersdorf.ContextMatching.Base.PositionTerm
import Gersdorf.ContextMatching.Base.HeapTerm
import Gersdorf.ContextMatching.Base.TermEquation
import Gersdorf.ContextMatching.Base.SimpleTermEquation

import Gersdorf.ContextMatching.Linear.LinearCM

import Gersdorf.ContextMatching.General.Rules.Split

corrPosLinearProvider :: (Term t) =>
  CorrPosProvider t HeapPath 
corrPosLinearProvider =
  cCorrPosProvider "correspondingLinear" correspondingPosLinear True

correspondingPosLinear :: (Term t) =>
    (t, t) 
    -> [(HeapPath, HeapPath)]
correspondingPosLinear =
	map (\(l, r) -> (getPath l, getPath r)) . correspondingTermsLinear

correspondingTermsLinear :: (Term t) => 
    (t, t)
    -> [(HeapTerm, HeapTerm)]
correspondingTermsLinear (l,r) =
    let heq :: SimpleTermEquation HeapTerm
        heq   = cEquation lht rht
        lht   = toHeapTerm l
        rht   = toHeapTerm r

        linEq = snd $ linearize 0 heq
        linL  = getLeft linEq
        linR  = getRight linEq
      
        dynTable =  let d = dynamicTableLinearCMDecision (lht, rht)
                    in  d
                    --in  trace (show d) d
        
        -- n: rows, first index
        n    = size linL
        -- m: cols, snd index
        m    = size linR

        getRow i    = map (\j -> dynamicSolve dynTable (i, j)) [1..m]
        --getColumn j = map (\i -> dynamicSolve dynTable (i, j)) [1..n]

        toIntList bs    = map (\b -> if (b) then 1 else 0) bs
        exactly1Checked is = ((foldl (+) 0 is) == 1)

        exactly1Row = exactly1Checked . toIntList . getRow
        --exactly1Col = exactly1Checked . toIntList . getColumn

        singleRows = filter exactly1Row [1..n]
        --singleCols = filter exactly1Col [1..m]
       
        getPair index selector builder = 
            map (\(i, j) -> (getAt lht i, getAt rht j)) $
            map (buildPair selector builder ) index
                where
                buildPair selector builder i = 
                    let j = fromJust (elemIndex True (selector i)) + 1
                    in  builder (i, j)
        
        --pairsCols = getPair singleCols getColumn (\(i, j) -> (j, i))
        pairsRows = getPair singleRows getRow id

	-- Wir betrachten nur Zeilen mit nur einem True. Spalten mit nur einem
	-- True sind nichtdeterministisch. Beispielsweise sind Zeilen mit
	-- Variablen in jeder Spalte mit einem True versehen, da eine Variable
	-- jeden Term matcht. Hat man nun zwei Spalten, in denen jeweils nur ein True
	-- in einer Zeile mit derselben Variablen steht, so bedeutet dies,
	-- dass die Variable entweder die eine oder die andere Spalte matcht.
	-- Beispielproblem hierfr: C[f(x)] =? h(f(a()), f(b())). In diesem
	-- Fall sieht die Ergebnistabelle so aus:
	--
	--                | h(..) | f(a()) | a() | f(b()) | b()
	--        --------+-------+--------+-----+--------+-----
	--        C[f(x)] |   +   |    +   |  -  |   +    |  -
	--        f(x)    |   -   |    +   |  -  |   +    |  -
	--        x       |   +   |    +   |  +  |   +    |  +
	--
	-- x wird durch a() oder b() gematcht. Die Lsung des Problems
        -- lautet: { {<h(_, f(b())/C>, <a()/x>}, { {<h(f(a()), _)/C>, <b()/x>} }.
        -- (x, a()) bzw. (x, b()) sind keine corresponding positions. Im Falle (x, a())
	-- lautet die Argumentation: In diesem Falle mssten C[f(_)]* = h(f(_), f(b()))*
        -- und x* = a()* fr alle Lsungssubstitutionen * gelten. Im Falle der
	-- ersten Substitution gilt dies auch, aber nicht im Falle der zweiten, da
	-- h(f(a()), _) /= h(f(_), f(b())) und b() /= a().
	pairs = pairsRows

    in  pairs


