module Gersdorf.ContextMatching.General.EnvEquationSet (
    EnvEquationSet(..), 
    EnvPosEqSet,
    SimpleEnvEquationSet,
    RuleResult(..),
    cRuleResult,
    module Gersdorf.ContextMatching.Base.EquationSet,
    module Gersdorf.ContextMatching.Base.Substitution
) where

import Gersdorf.ContextMatching.Base.EquationSet
import Gersdorf.Data.TreeFiniteMap 
import Gersdorf.ContextMatching.Base.Substitution
import Gersdorf.ContextMatching.Base.PositionTerm
import Gersdorf.Data.List
import Gersdorf.Data.BacktrackingChoice

data (Term t, TermEquation e t) => 
    RuleResult e t = RRNothing 
                     | RRBot 
                     | RResult {
                         rrNewEquations :: [e t],
                         rrSolved       :: [(TermType, t)]
                       } deriving Show

cRuleResult :: (TermEquation e t) =>
    [e t] -> [(TermType, t)] -> RuleResult e t
cRuleResult eq sub = RResult { rrNewEquations = eq, rrSolved = sub }

class (EquationSet es eq t, SubstitutionSet (es eq) t) => EnvEquationSet es eq t where
   scanUpdate :: es eq t
    -> (eq t -> [BacktrackingChoice (RuleResult eq t)])
    -> [BacktrackingChoice (es eq t)]


class (EnvEquationSet es eq t, PositionTerm t tp, AnalysisTerm t,
    TreeExtractor t, TermBuilder t, 
    SubstitutionGenerator (es eq t) t) 
    => EnvPosEqSet es eq t tp

instance (AnalysisTerm t, TermEquation eq t, PositionTerm t tp, 
          TreeExtractor t, TermBuilder t)  
          => EnvPosEqSet SimpleEnvEquationSet eq t tp

data (TermEquation e t) => SimpleEnvEquationSet e t = 
    SimpleEnvEquationSet ([e t], TreeFiniteMap TermType t)
    | SimpleEnvBot
	deriving Eq

instance (TermEquation e t) => Show (SimpleEnvEquationSet e t) where
    showsPrec p (SimpleEnvEquationSet (l, e)) r = showString "<" (shows l (showString " :: " (shows e (showString ">" r))))
    showsPrec _ SimpleEnvBot r = showString "<bot>" r

instance (TermEquation e t) => EquationSet SimpleEnvEquationSet e t where
    emptySet = SimpleEnvEquationSet ([], emptyFM) 
    cBotSet  = SimpleEnvBot
    cSet l = SimpleEnvEquationSet (l, emptyFM)
    
    addToSet (SimpleEnvEquationSet (s, sub)) eq = SimpleEnvEquationSet (eq:s, sub)
    addToSet SimpleEnvBot _                     = SimpleEnvBot
    
    isEmpty (SimpleEnvEquationSet (s, _)) = null s
    isEmpty SimpleEnvBot                  = False
    
    isEqBot (SimpleEnvEquationSet _)      = False
    isEqBot SimpleEnvBot                  = True
    
    toEquationList (SimpleEnvEquationSet (s, _)) = s
    toEquationList SimpleEnvBot           = []

instance (TermEquation e t) => EnvEquationSet SimpleEnvEquationSet e t where
    scanUpdate (SimpleEnvEquationSet (s, sub)) f = scanUpdate' s f
        where
        scanUpdate' eqs f = concat $ map update (removeEach eqs)
        update (otherEqs, eq) = map (fmapRemove (filterIt otherEqs)) (f eq)

        --filterIt :: (TermEquation e t) => [e t] -> RuleResult e t -> Maybe (SimpleEnvEquationSet e t)
    	filterIt _    RRNothing            = Nothing                                   -- Lschen aus Liste
        filterIt _    RRBot                = Just SimpleEnvBot                         -- Wird botListe
    	filterIt 
            oEqs (RResult { rrNewEquations = newEqs,
                            rrSolved = solved }) =  
                                             let newListSet = newEqs ++ oEqs
                                                 newEnv     = updateEnv sub solved
                                             in  Just (SimpleEnvEquationSet (newListSet, newEnv))  -- Rckgabe: alte Liste zzgl. neue Gleichungen incl. Zusatzwert!

        updateEnv oldEnv []           = oldEnv
        updateEnv oldEnv ((tt, t):ss) = bindFM tt t (updateEnv oldEnv ss)
                                             
    scanUpdate SimpleEnvBot _ = []

instance (Term t, TermEquation e t) => SubstitutionSet (SimpleEnvEquationSet e) t where
    emptySubstitution = SimpleEnvEquationSet ([], emptyFM)

    lookupSubstitution (SimpleEnvEquationSet (_, sub)) t = lookupFM t sub

    bindSubstitution (SimpleEnvEquationSet (s, sub)) t tt = 
        SimpleEnvEquationSet (s, bindFM t tt sub)

instance (TermEquation e t) => SubstitutionGenerator (SimpleEnvEquationSet e t) t where
    generate SimpleEnvBot = []
    generate (SimpleEnvEquationSet (_, sub)) = [Substitution (map convertToSubst (fmToList sub))]
        where
        convertToSubst (tt, t)
            | isVariable tt        = SubstVar tt t
            | isContextVariable tt = SubstContextVar tt t
            | otherwise            = error "EnvEquationSet.generate: No Termtype as ContextVar. May not happen."
