module Gersdorf.ContextMatching.General.Heuristic (
    HeuristicFunction,
    HeuristicChoice(..),
    Strategy(..),
    runStrategy,
    take1Heuristic,
    module Gersdorf.ContextMatching.General.Rule
) where

import Debug.Trace

import Gersdorf.ContextMatching.General.Rule
import Gersdorf.Data.BacktrackingChoice
import Gersdorf.ContextMatching.Base.Utilities

-- | Eine HeuristicFunction nimmt eine Menge von mglichen anwendbaren
-- Regeln und whlt aus dieser Menge eine aus, die angewendet werden soll.
type HeuristicFunction = (EnvPosEqSet es eq t tp) =>
    [RuleApplicationResult es eq t tp] 
    -> HeuristicChoice es eq t tp

data (EnvPosEqSet es eq t tp) => HeuristicChoice es eq t tp =
	HeuristicChoice {
	   chosenRule   :: Rule es eq t tp,
	   chosenResult :: BacktrackingChoice (es eq t)
 	}
	deriving (Show)

data (EnvPosEqSet es eq t tp) => Strategy es eq t tp = Strategy {
    strategyName :: String,
    strategyHeuristic :: HeuristicFunction,
    strategyRules :: [Rule es eq t tp]
}

instance (EnvPosEqSet es eq t tp) => Show (Strategy es eq t tp) where
    showsPrec _ str r = showPredList (strategyName str ++ " [") (strategyRules str) ", " " ]" r

runStrategy :: (EnvPosEqSet es eq t tp) =>
	Strategy es eq t tp
	-> es eq t
	-> HeuristicChoice es eq t tp
runStrategy str eset = 
    let appliedResults = map (applyRule eset) (strategyRules str)
        -- alle diejenigen RuleApplicationResults rausfiltern, die nicht ein Ergebnis haben
        filterEmptyResults = filter (any (not . noChoice) . appliedResult) appliedResults
    in  (strategyHeuristic str) appliedResults

take1Heuristic rs = 
    let filterEmptyRules = filter (any (not . noChoice) . appliedResult) rs
        firstResult = head $ filterEmptyRules
        resultChoice = 
            HeuristicChoice {
        	   chosenRule   = appliedRule firstResult,
        	   chosenResult = head $ appliedResult firstResult
            }
    in  resultChoice
