module Gersdorf.ContextMatching.General.Rules.ConstantEliminate (
    constantEliminateRule
) where

import Data.Maybe

import Gersdorf.Data.Tuple
import Gersdorf.Data.BacktrackingChoice

import Gersdorf.ContextMatching.Base.TermParser
import Gersdorf.ContextMatching.Base.Terms

import Gersdorf.ContextMatching.Base.SimpleTermEquation
import Gersdorf.ContextMatching.Base.PositionTerm

import Gersdorf.ContextMatching.General.Rule
import Debug.Trace

constantEliminateRule :: (EnvPosEqSet es eq t tp) => Rule es eq t tp
constantEliminateRule = cSimpleRule1 "constantEliminate" constantEliminate

constantEliminate :: (EnvPosEqSet es eq t tp) => 
    es eq t 
    -> eq t 
    -> RuleResult eq t
constantEliminate _ eq
        | isApplicable =
               let positions    = map (scalarApply getPath) constantPairs
                   (newl, newr) = createHoledTerm (getPair eq) positions
               in  cRuleResult [cEquation newl newr] []

        | otherwise    = RRNothing -- rule not applicable
        
        where
        isApplicable :: Bool
        isApplicable = not (null constantPairs)

        constantPairs = getMatchingConstants eq

createHoledTerm :: (PositionTerm t tp, SubTermBuilder t) =>
    (t, t)
    -> [(tp, tp)]
    -> (t, t)
createHoledTerm ts [] = ts
createHoledTerm (l,r) ((tpl, tpr):tps) =
    let newl = fromJust $ replacePath l tpl (cHole)
        newr = fromJust $ replacePath r tpr (cHole)
    in  createHoledTerm (newl, newr) tps


eq :: ContextTermEq
eq = read "X[f(Y[a()], Y[g(c())])] =? f(g(b()), f(a(), g(c())))"

