module Gersdorf.ContextMatching.General.Rules.ContextElimination (
    ctxEliminationRule
) where

import Gersdorf.ContextMatching.General.Rule
import Gersdorf.ContextMatching.Base.PositionTerm

import Debug.Trace 

-- | Regel wie folgt:
-- { X =? C[_] && X[s] =? t } -> { X =? C[_] && s =? t' }
--                               (falls s != _ && C[t'] =? t)
--
ctxEliminationRule :: (EnvPosEqSet es eq t tp) => Rule es eq t tp
ctxEliminationRule = cSimpleRule1 "ctxElimination" ctxElimination

ctxElimination :: (EnvPosEqSet es eq t tp) => es eq t -> eq t -> RuleResult eq t
ctxElimination eset eq
        | isApplicable
          
          = let maybe_sol   = do eq_C     <- lookupSubstitution eset $ getType eq_X
                                 holePath <- getHolePath eq_C
                                 eq_t'    <- gotoPath eq_t holePath
                                 eq_C_t'  <- instantiateContext eq_C (extractNode eq_t')
                                 
                                 let eq_s_extr  = extractSubTerm eq_s
                                     eq_t'_extr = extractSubTerm eq_t'

                                 if (eq_C_t' /= eq_t) 
                                    then Nothing
                                    else return (cRuleResult [cEquation eq_s_extr eq_t'_extr] [])
                                    
            in  case (maybe_sol) of
                    Nothing    -> RRBot
                    (Just sol) -> sol
                    
        | otherwise                    = RRNothing
    
        where
        -- initialization of rule variables
        eq_X = getLeft eq
        eq_s = head $ getChildren eq_X
        eq_t = getRight eq
        
        isApplicable :: Bool
        isApplicable = isContextVariable eq_X 
                       && hasSubstitution eset (getType eq_X)
                       && not (isHole eq_s)

