module Gersdorf.ContextMatching.Linear.LinearityTest (
    extractTerms,
    isLinear
) where

import Gersdorf.ContextMatching.Base.UnificationProblem
import Gersdorf.ContextMatching.Base.Term
import Gersdorf.Data.Trie 
import Debug.Trace

-- | Testet, ob das gegebene Unifikationsproblem linear ist, d. h.,
-- ob jede Variable und Kontextvariable hchstens einmal im Problem auftaucht.
-- Dabei wird in der Implementierung ein Trie verwandt, um bereits vorgekommene
-- Variablen zu merken.
isLinear :: (UnificationProblem up e t) => up e t -> Bool
isLinear up = (checkDupes (\t -> isVariable t) getName terms 
               && checkDupes (\t -> isContextVariable t) getName terms)
   where terms = extractTerms up

-- | Die Funktion nimmt eine Termliste und prft, ob ein Term mehrfach vorkommt.
-- Dazu bentigt sie eine Filterfunktion (erstes Argument), die angibt, ob ein
-- bestimmter Term der Termliste berhaupt bercksichtigt werden soll. Weiterhin
-- wird eine Transformationsfunktion (zweites Argument) verlangt, die einen Term
-- der Liste in einen String umwandelt. Diese Stringreprsentation wird verwendet,
-- um auf Duplikate zu prfen.
checkDupes :: (TermIterator t) => 
    (t -> Bool)        -- ^ Testfunktion, die angibt, ob ein bestimmter Term geprft werden soll
    -> (t -> String)   -- ^ Tranformationsfunktion, um einen Term in einen String umzuwandeln
    -> [t]             -- ^ Liste der Terme, die auf Duplikate geprft werden
    -> Bool            -- ^ Gibt es einen Term, der mehrfach vorkommt?
checkDupes fil f ts = checkDupes' fil f emptyFM (filter fil ts)
    where
    checkDupes' :: (t -> Bool) ->(t -> String) -> StringTrie () -> [t] -> Bool
    checkDupes' fil f _ [] = True
    checkDupes' fil f tr (t:ts) =  --trace ("checkDupes'' with t=" ++ (show $ f t))
                                    (case (lookupFM (f t) tr) of
                                           Just _ -> False
                                           Nothing -> checkDupes' fil f (bindFM (f t) () tr) ts)

-- | Extrahiert alle Terme eine Unifikationsproblems, und zwar sowohl von der
-- linken wie von der rechten Seite.
extractTerms :: (UnificationProblem up e t) => up e t -> [t]
extractTerms up = concat $ map toList (leftTerms ++ rightTerms)
  where termEquationList = toEquationList up
        leftTerms = map getLeft termEquationList
        rightTerms = map getRight termEquationList

