module Gersdorf.ContextMatching.Test.StatisticTests (
    createStatistics,
    EnvPosEqSet,
    SimpleEnvEquationSet
) where

import Debug.Trace
import Data.List
import Control.Monad

import Gersdorf.Data.TreeFiniteMap
import Gersdorf.Monad.IOUtil

import Gersdorf.ContextMatching.General.GeneralFacade
import Gersdorf.ContextMatching.General.EnvEquationSet
import Gersdorf.ContextMatching.General.Heuristic

import Gersdorf.ContextMatching.Base.UnificationProblem
import Gersdorf.ContextMatching.Base.TermGenerator
import Gersdorf.ContextMatching.Base.Utilities

type RuleMapping es eq t tp = [(Int, Rule es eq t tp)]

{-
createStatistics :: (UnificationProblem up eq t, Substitutable (up eq t),
                   (EnvPosEqSet SimpleEnvEquationSet eq t tp),
                   EquationSet es eq t) =>
    [Strategy SimpleEnvEquationSet eq t tp] -- ^ Anzuwendende Strategien
    -> (IO [up eq t])                       -- ^ Termgenerator
    -> Int                                  -- ^ Anzahl Testsets
    -> [IO String]                          -- ^ Ergebnis als Stringarray (je Zeile ein String)
-}
createStatistics strs tg n =
    let uniqueRules = nub (concat $ map strategyRules strs)
        ruleMapping = zip [1..] uniqueRules
        header      = formatHeader ruleMapping
        statistics  = map (\n -> createStatistic strs ruleMapping (n, tg)) [1..n]

    in  (return header):statistics
{-    
    replicate n (createStatistics' strs tg)
        
    let uniqueRules = nub (concat $ map strategyRules strs)
        ruleMapping = zip [1..] uniqueRules
    in  do let tgs    = replicate n tg
               iorows = map (createStatistic strs ruleMapping) (zip [1..] tgs)
           rows <- sequenceG iorows
           return ((formatHeader ruleMapping):rows)
-}
createStatistic :: (UnificationProblem up eq t, Substitutable (up eq t),
                   (EnvPosEqSet SimpleEnvEquationSet eq t tp)) => 
    [Strategy SimpleEnvEquationSet eq t tp] -- ^ Anzuwendende Strategien
    -> RuleMapping SimpleEnvEquationSet eq t tp -- ^ RuleMapping
    -> (Int, IO [up eq t])                   -- ^ Samplenummer
    -> IO String                             -- ^ Ergebnis als String
createStatistic strs ruleMapping (sampleno, tg) =
    do ups <- tg
       return $ concat $ map (createStatistic' ups) strs 
    where
    createStatistic' ups str =
        let asi = (str, solveGCMAggregatedStatistic str ups)
        in  formatSample ruleMapping sampleno asi

formatHeader :: (EnvPosEqSet SimpleEnvEquationSet eq t tp) =>
   RuleMapping SimpleEnvEquationSet eq t tp  -- ^ RuleMapping
   -> String
formatHeader ruleMapping =
    let header = 
            "Sample; "
            ++ "Strategie; "
            ++ "avg. problem size; "
            ++ "total solutions; "
            ++ "total steps; "
            ++ "avg steps; "
            ++ "avg function count; "
            ++ "avg diff function count; "
            ++ "avg hole count; "
            ++ "avg variable count; "
            ++ "avg diff variable count; "
            ++ "avg cvariable count; "
            ++ "avg diff cvariable count; "
            ++ showPredList "" (map snd ruleMapping) "; " "" "\n"
    in  header

formatSample :: (EnvPosEqSet es eq t tp) =>
    RuleMapping SimpleEnvEquationSet eq t tp  -- ^ RuleMapping
    -> Int                                     -- ^ Samplenummer
    -> (Strategy es eq t tp, AggregatedStatisticInformation es eq t tp)
    -> String
formatSample ruleMapping sampleno asiPair =
    let str = fst asiPair
        asi = snd asiPair
        sortedRuleStepResults = sortRuleStepResults ruleMapping asi
        resultstr = show sampleno ++ "; "
                    ++ strategyName str ++ "; "
                    ++ show (asiAvgSize asi) ++ "; "
                    ++ show (asiTotalSolutions asi) ++ "; "
                    ++ show (asiTotalSteps asi) ++ "; "
                    ++ show (asiAvgSteps asi) ++ "; "
                    ++ show (asiAvgFunctionCount asi) ++ "; "
                    ++ show (asiAvgDifferentFunctionCount asi) ++ "; "
                    ++ show (asiAvgHoleCount asi) ++ "; "
                    ++ show (asiAvgVariableCount asi) ++ "; "
                    ++ show (asiAvgDifferentVariableCount asi) ++ "; "
                    ++ show (asiAvgCVariableCount asi) ++ "; "
                    ++ show (asiAvgDifferentCVariableCount asi) ++ "; "
                    ++ showPredList "" sortedRuleStepResults "; " "" ""
                    ++ "\n"
    in resultstr

-- | sortiert die Rulestep-Ergebnisse aus der AggregatedStatisticInformation
--   entsprechend des Rulemappings.
sortRuleStepResults :: (EnvPosEqSet es eq t tp) =>
    RuleMapping SimpleEnvEquationSet eq t tp  -- ^ RuleMapping
    -> AggregatedStatisticInformation es eq t tp
    -> [Int]
sortRuleStepResults ruleMapping asi =
    let noRules = length ruleMapping
        rules   = map snd ruleMapping

        ruleNameMap = map (\(rule, i) -> (ruleName rule, i)) (asiRuleSteps asi)
        
        lookupMap :: TreeFiniteMap String Int
        lookupMap = listToFM ruleNameMap

        lookupRule r = 
            case (lookupFM (ruleName r) lookupMap) of
                Nothing  -> 0
                (Just i) -> i
    in  map lookupRule rules

---------------------------------------------------------------------
-- Statistical Helping routines
---------------------------------------------------------------------

