module Gersdorf.ContextMatching.Test.TestStrategies where

import Gersdorf.ContextMatching.General.Rules.Rules
import Gersdorf.ContextMatching.General.Heuristic

--------------------------------------------------------------------------------------
-- Regeldeklaration
--------------------------------------------------------------------------------------

strategies :: (EnvPosEqSet SimpleEnvEquationSet eq t tp) =>
    [Strategy SimpleEnvEquationSet eq t tp]
strategies = [extended2Strategy, extendedStrategy, paperStrategy, noSplitStrategy, baseStrategy]

-- | Alle Regeln inkl. Erweiterungen
extended2Strategy :: (EnvPosEqSet SimpleEnvEquationSet eq t tp) => 
    Strategy SimpleEnvEquationSet eq t tp
extended2Strategy = Strategy {
      strategyName      = "extended2Strategy",
	  strategyHeuristic = take1Heuristic,
	  strategyRules     = [ {- Hilfsregeln -}
                           solvedDummyRule, botRemoveRule, 

                           {- deterministische Regeln -}
                           decomposeRule, holeDecomposeRule, 
                           ctxEliminationRule, multiCtxCollisionRule, 
                           constantEliminateRule,
                           multiContextDecomposeRule,

                           {- Splitrules -}
                           splitRule corrPosFunctionsProvider,
                           splitRule corrPosHolesProvider,
                           splitRule corrPosLinearProvider,

                           {- nichtdeterministische Regeln -}
                           bottomDecomposeRule, variableSplitRule]
	}

-- | Alle Regeln inkl. Erweiterungen
extendedStrategy :: (EnvPosEqSet SimpleEnvEquationSet eq t tp) => 
    Strategy SimpleEnvEquationSet eq t tp
extendedStrategy = Strategy {
      strategyName      = "extendedStrategy",
	  strategyHeuristic = take1Heuristic,
	  strategyRules     = [ {- Hilfsregeln -}
                           solvedDummyRule, botRemoveRule, 

                           {- deterministische Regeln -}
                           decomposeRule, holeDecomposeRule, 
                           ctxEliminationRule, multiCtxCollisionRule, 
                           constantEliminateRule,
                           multiContextDecomposeRule,

                           {- Splitrules -}
                           splitRule corrPosHolesProvider,
                           splitRule corrPosLinearProvider,

                           {- nichtdeterministische Regeln -}
                           bottomDecomposeRule, variableSplitRule]
	}

-- | Alle Regeln aus dem Originalpapier
paperStrategy :: (EnvPosEqSet SimpleEnvEquationSet eq t tp) => 
    Strategy SimpleEnvEquationSet eq t tp
paperStrategy = Strategy {
      strategyName      = "paperStrategy",
	  strategyHeuristic = take1Heuristic,
	  strategyRules     = [ {- Hilfsregeln -}
                           solvedDummyRule, botRemoveRule, 

                           {- deterministische Regeln -}
                           decomposeRule, holeDecomposeRule, 
                           ctxEliminationRule, multiCtxCollisionRule, 
                           multiContextDecomposeRule,

                           {- Splitrules -}
                           splitRule corrPosLinearProvider,

                           {- nichtdeterministische Regeln -}
                           bottomDecomposeRule, variableSplitRule]
	}

-- | alle Regeln ohne die Splitregeln
noSplitStrategy :: (EnvPosEqSet SimpleEnvEquationSet eq t tp) => 
    Strategy SimpleEnvEquationSet eq t tp
noSplitStrategy = Strategy {
      strategyName      = "noSplitStrategy",
	  strategyHeuristic = take1Heuristic,
	  strategyRules     = [ {- Hilfsregeln -}
                           solvedDummyRule, botRemoveRule, 


                           {- deterministische Regeln -}
                           decomposeRule, holeDecomposeRule, 
                           ctxEliminationRule, multiCtxCollisionRule, 
                           multiContextDecomposeRule,

                           {- nichtdeterministische Regeln -}
                           bottomDecomposeRule, variableSplitRule]
	}

-- | alle Regeln ohne Regeln, die auf korrespondierenden Positionen basieren
baseStrategy :: (EnvPosEqSet SimpleEnvEquationSet eq t tp) => 
    Strategy SimpleEnvEquationSet eq t tp
baseStrategy = Strategy {
      strategyName      = "baseStrategy",
	  strategyHeuristic = take1Heuristic,
	  strategyRules     = [ {- Hilfsregeln -}
                           solvedDummyRule, botRemoveRule, 

                           {- deterministische Regeln -}
                           decomposeRule, holeDecomposeRule, 
                           ctxEliminationRule, multiCtxCollisionRule, 

                           {- nichtdeterministische Regeln -}
                           variableSplitRule]
	}


