-- | Eine Assoziationsdatentyp, der auf einer Liste definiert ist.
module Gersdorf.Data.AssocList (
    AssocList,
    module Gersdorf.Data.FiniteMap
) where

import Gersdorf.Data.FiniteMap
import Data.List

-- | Datentyp, der eine Liste von assoziationen reprsentiert. Dabei ist @k@ der
-- Datentyp der Schlssels (Key) und @a@ der Datentyp des Assoziationsziels.
data AssocList k a = AssocList [(k, a)]
    deriving Show

-- | Eine Assoziazionsliste ist ein FiniteMap ber einem Schlssel k.
instance (Eq k) => FiniteMap AssocList k where
    emptyFM = AssocList []
    listToFM l = AssocList l
    fmToList (AssocList l) = l

    lookupFM k (AssocList l) = {-# SCC "lookup Assoclist" #-}
                             Data.List.lookup k l

    bindFM k a (AssocList l) = {-# SCC "bind AssocList" #-}
                             AssocList (bind' k a l)
        where bind' kn a [] = [(k, a)]
              bind' kn a (f@(k, _):ll) = if (kn == k) then ((kn, a):ll)
                                                    else f:(bind' kn a ll)


