module Gersdorf.Data.BacktrackingChoice (
  BacktrackingChoice,
  isBacktrackable,
  isBTOne, isBTMany, isBTDet,
  getChoices,
  topChoice, popChoice,
  noChoice,
  cBacktrackingOne,
  cBacktrackingMany,
  cBacktrackingDet,
  cBacktrackingNo,
  module Gersdorf.Data.List
) where

import Gersdorf.Data.List
import Gersdorf.ContextMatching.Base.Utilities

data BTType = BTOne | BTMany | BTDet
    deriving Eq

data BacktrackingChoice a = 
    BTChoice {
        btType    :: BTType,
        btEntries :: [a]
    }

instance (Show a) => Show (BacktrackingChoice a) where
    showsPrec _ b r 
      | (btType b == BTDet)  = showString "{! " (shows (topChoice b) (showString " !}" r))
      | (btType b == BTMany) = showPredList "{ " (btEntries b) ", " " }" r
      | (btType b == BTOne)  = showPredList "{ " (btEntries b) "/ " " }" r

isBTMany :: BacktrackingChoice a -> Bool
isBTMany b = (btType b == BTMany)

isBTOne :: BacktrackingChoice a -> Bool
isBTOne b = (btType b == BTOne)

isBTDet :: BacktrackingChoice a -> Bool
isBTDet b = (btType b == BTDet)

isBacktrackable :: BacktrackingChoice a -> Bool
isBacktrackable b
    | (btType b == BTDet)  = False
    | (null (btEntries b)) = False
    | otherwise            = True

noChoice :: BacktrackingChoice a -> Bool
noChoice b
    | (null (getChoices b)) = True
    | otherwise             = False

getChoices :: BacktrackingChoice a -> [a]
getChoices = btEntries

topChoice :: BacktrackingChoice a -> Maybe a
topChoice b
    | null chs  = Nothing
    | otherwise = Just (head chs)
    where
    chs = getChoices b

popChoice :: BacktrackingChoice a -> BacktrackingChoice a
popChoice b
    | noChoice b = b
    | otherwise  = b { btEntries = tail (btEntries b) }

--
-- smart constructors
--
cBacktrackingMany :: [a] -> BacktrackingChoice a
cBacktrackingMany l = BTChoice { btType = BTMany, btEntries = l }

cBacktrackingOne :: [a] -> BacktrackingChoice a
cBacktrackingOne l = BTChoice { btType = BTOne, btEntries = l }

cBacktrackingDet :: a -> BacktrackingChoice a
cBacktrackingDet a = BTChoice { btType = BTDet, btEntries = [a] }

cBacktrackingNo :: BacktrackingChoice a
cBacktrackingNo = BTChoice { btType = BTMany, btEntries = [] }

instance Functor BacktrackingChoice where
    fmap f b = b { btEntries = map f (btEntries b) }

instance FunctorRemove BacktrackingChoice where
    fmapRemove f b = b { btEntries = mapRemove f (btEntries b) }

