module Gersdorf.Data.ContextTree (
    ContextTree,
    module Gersdorf.Data.Tree,
    module Gersdorf.Data.StandardTreePath
) where

import Data.Array

import Gersdorf.Data.Tree
import Gersdorf.Data.StandardTreePath
import Gersdorf.ContextMatching.Base.Utilities
import Gersdorf.Data.List

data ContextTree c a = ContextTree {
    value :: a,
    context :: c,
    children :: Array Int (ContextTree c a),

    -- ab hier: Verwaltungsinformationen!
    father :: Maybe (ContextTree c a),
    childIndex :: Int,
    path :: StandardTreePath,

    contextFunction :: ContextTree c a -> c
    } 

-- | Wir mssen die Instanz von Eq selbst definieren, damit es wegen der
-- father-Beziehung zu keiner endlosen Rekursion kommt!
instance (Ord a) => Eq (ContextTree c a) where
    l == r = (value l == value r)
             && (all (foldPair (==)) (zip (getChildren l) (getChildren r)))

instance (Ord a) => Ord (ContextTree c a) where
    l < r = lessTree l r
    l <= r = (l < r) || (l == r)

{-
instance (Show a) => Show (ContextTree a) where
    showsPrec _ n@(ContextTree { value = v, 
                                  father = f, 
                                  children = chs }) r
        = let chList = getChildren n
        
              fatherShow :: String
              fatherShow = case (f) of
                                Nothing -> ""
                                (Just father) -> " father=\"" ++ (show (value father)) ++ "\""
          in case (chList) of
                [] -> showString ("<node value=\"" ++ (show v) ++ "\"" ++ fatherShow ++ "/>") r
                (x:xs) -> showPredList ("<node value=\"" ++ (show v) ++ "\"" ++ fatherShow ++ ">\n") chList "\n" "\n</node>\n" r
-}

{-
instance  Functor (ContextTree c) where
    fmap f t = 
        let newnode = t { value    = f (value t),
                          children = listArray (1, arity t) (map (fmap f) (getChildren t)),
                          context  = (contextFunction t) newnode 
                        }
        in  newnode
-}                    

instance (Ord a) => Listable (ContextTree c a) where 
    toList = toListTree

instance (Ord a) => Flatable (ContextTree c) a where
    flatten = map getContent . toList

instance (Ord a) => Tree (ContextTree c a) where
    getChildren = elems . children
    arity = snd . bounds . children
    getChild t i  | (i < 1 || i > arity t) = Nothing
                  | otherwise              = Just $ children t ! i

instance Container (ContextTree c) a where
    getContent t = value t

instance (Ord a) => ContainerTree (ContextTree c) a 

instance (Ord a) => TreeExtractor (ContextTree c a) where
    extractNode t = {-# SCC "extractNode" #-} cNodeC (getContent t) (contextFunction t) (map extractNode (getChildren t))

    replaceChildren t css = cNodeC (getContent t) (contextFunction t) css

instance (Ord e) => ContextableTree ContextTree c e where
    getContext t = context t
    getContextFunction t = contextFunction t


instance (Ord e) => ContextTreeBuilder ContextTree c e where
    cNodeC v ctf cs f = let node = ContextTree {
                                    value = v,
                                    context = ctf node,
                                    contextFunction = ctf,
                                    father = case (f) of
                                                Nothing -> Nothing
                                                (Just (father, _)) -> Just father,
                                    children = listArray (1, length cs) 
                                                         (map (\ (etcc, n) -> etcc (Just (node, n))) (zip cs [1..])),
                                    childIndex = case (f) of
                                                   Nothing         -> 0
                                                   (Just (_, nch)) -> nch,
                                    path = case (f) of
                                              Nothing              -> mkPath []
                                              (Just (father, nch)) -> mkPath ((toPathIndexList $ getPath father) ++ [nch])
                                    }
                   in node

instance (Ord a) => UpdatableTree (ContextTree c a) StandardTreePath where
    transformTreePath f r tp = cTree $ trfTreePath' r tp 0
        where
        trfTreePath' cur tp dp= let mbNewNode = f cur dp
                                in  case (mbNewNode) of
                                      Nothing ->
                                        if (isEmptyPath tp) then
                                            extractNode cur 
                                        else
                                            let nextChild = children cur ! nextChildIdx
                                                nextChildIdx = topIndex tp
                                                newChildrenL = mapNth (\_ -> trfTreePath' nextChild (popPath tp) (dp + 1))
                                                                      (\c -> (extractNode c))
                                                                      nextChildIdx
                                                                      (getChildren cur)
                                            in cNodeC (getContent cur) (contextFunction cur) newChildrenL
                                                             
                                      (Just newNode) -> newNode
                                        
                      

    replaceSubtree ct nnode 
        | isRoot ct = cTree nnode -- falls der ChildTree bereits root war -> nct wird auch root
        | otherwise = transformTreePath (replaceLeaf nnode) (getRoot ct) ctPath

        where
        ctPath = getPath ct
        replaceLeaf nnode curnode depth = if (depth == pathDepth ctPath) then Just nnode
                                                                         else Nothing

instance (Ord a) => NavigatableTree (ContextTree c a) where
    getFather t = father t


instance (Ord a) => AdressableTree (ContextTree c a) StandardTreePath where
    cPath t itp = let tp = mkPath itp
                  in  case (gotoPath t tp) of
                        Nothing  -> Nothing
                        (Just _) -> Just tp

    getPath = path

    getChildIndex t = childIndex t

instance (Ord a) => ContainerNavigatableTree (ContextTree c) a 

instance (Ord a) => ContainerAdressableTree (ContextTree c) a StandardTreePath

{-instance (ContainerTree t1 e) => ConvertableContainerTree t1 (ContextTree c) e where
    convertTree = convertTree'
        where 
        convertTree' t1 = let v  = getContent t1
                              cs = getChildren t1
                          in  cNodeC v (\_ -> 3) (map convertTree' cs)
-}
