-- | Dieses Modul beinhaltet die Typklasse FiniteMap, wie sie in
-- Okasaki, Purely functional Data Structure beschrieben ist.
module Gersdorf.Data.FiniteMap (
    FiniteMap(..)
) where

import Data.Maybe (isJust)

-- | Eine FiniteMap m ist eine Assoziation ber einen Schlssel k. Der Typ a ist dabei
-- der Typ des Assoziationsziels.
--
-- Aus Okasaki, Purley Functional Data Structures
class (Eq k) => FiniteMap m k where
    -- | Erzeugt eine leere FiniteMap.
    emptyFM :: m k a

    -- | Erzeugt eine FiniteMap aus der Liste
    listToFM :: [(k, a)] -> m k a

    -- | Gibt eine Liste von Key/Value-Pairs zurck
    fmToList :: m k a -> [(k, a)]
    
    -- | Binden eines Assoziationsziels an einen Schlssel.
    bindFM :: k     -- ^ Der Schlssel
        -> a      -- ^ Das Assoziationsziel
        -> m k a  -- ^ Die FiniteMap, in der das Schlssel/Ziel-Paar gebunden werden soll
        -> m k a  -- ^ Ergebnis-FiniteMap, in der k und a gebunden sind.
    
    -- | Suchen eines Assoziationsziels ber einen Schlssel.
    lookupFM :: k    -- ^ Der Schlssel, mit dem gesucht wird
        -> m k a   -- ^ Die FiniteMap, in der gesucht wird.
        -> Maybe a -- ^ Just a, falls (k,a) in der Map gebunden sind, Nothing sonst.

    -- | Prft, ob ein Schlssel in der FiniteMap enthalten ist.
    containsFM :: k  -- ^ Schlssel, nach dem gesucht wird.
        -> m k a   -- ^ Die FiniteMap, in der gesucht wird.
        -> Bool    -- ^ True, falls Schlssel in der FiniteMap enthalten
    containsFM k m = isJust $ lookupFM k m

    updateFM :: k    -- ^ Schlssel, nach dem gesucht wird.
        -> a         -- ^ Element, das bei nichtfinden von k eingefgt wird
        -> (a -> a)  -- ^ Funktion, die den neuen Eintrag von k berechnet
        -> m k a     -- ^ FiniteMap, auf der die Operation luft
        -> m k a     -- ^ ErgebnisMap
    updateFM k nullEl f m =
        case (lookupFM k m) of
            Nothing  -> bindFM k nullEl m
            (Just x) -> bindFM k (f x) m
        
