module Gersdorf.Data.SimpleTree (
    SimpleTree,
    module Gersdorf.Data.Tree
) where

import Gersdorf.Data.Tree
import Gersdorf.ContextMatching.Base.Utilities
import Gersdorf.Data.List

data SimpleTree a = SimpleTree {
    value :: a,
    children :: [SimpleTree a]
    }

instance (Ord a) => Eq (SimpleTree a) where
    l == r = (value l == value r)
             && (all (foldPair (==)) (zip (getChildren l) (getChildren r)))

instance (Ord a) => Ord (SimpleTree a) where
    l < r = lessTree l r
    l <= r = (l < r) || (l == r)

{-
instance (Show a) => Show (SimpleTree a) where
    showsPrec _ t r = showPredList ((show $ getContent t) ++ "(") (getChildren t) "," ")" r
-}

instance (Ord a) => Listable (SimpleTree a) where
    toList t = t:(convertList (getChildren t))
        where
        convertList :: (Tree t) => [t] -> [t]
        convertList = concat . map toList

instance (Ord a) => Flatable SimpleTree a where
    flatten = map getContent . toList

instance (Ord a) => Tree (SimpleTree a) where
    getChildren t = children t

instance Container SimpleTree a where
    getContent t = value t

instance (Ord a) => ContainerTree SimpleTree a

instance (Ord e) => TreeExtractor (SimpleTree e) where
    extractNode t = cNode (getContent t) (map extractNode (getChildren t))

    replaceChildren t css = cNode (getContent t) css
    
instance (Ord e) => TreeBuilder SimpleTree e where
    cNode val chs f = let node = SimpleTree {
                                     value = val,
                                     children = (map (\(tn, n) -> tn (Just (node, n))) (zip chs [1..]))
                                  }
                      in  node


instance (Ord e, ContainerTree t1 e) => ConvertableContainerTree t1 SimpleTree e where
    convertTree = convertTree'
        where 
        convertTree' t1 = let v = getContent t1
                              cs = getChildren t1
                          in  cNode v (map convertTree' cs)
