module Gersdorf.Data.Trie (
    Trie,
    StringTrie,
    module Gersdorf.Data.FiniteMap
) where

import Gersdorf.Data.FiniteMap
import Gersdorf.Data.AssocList

data Trie mk ks a = Trie (Maybe a) (mk (Trie mk ks a))

instance FiniteMap m k => FiniteMap (Trie (m k)) [k] where
    emptyFM = Trie Nothing emptyFM

    listToFM [] = emptyFM
    listToFM ks = listToFM' ks (emptyFM :: Trie (m k) [k] a)
        where
        listToFM' ((k, e):ks) tr = listToFM' ks (bindFM k e tr)

    fmToList t = fmToList' [] t
        where
        fmToList' buf (Trie b m) = 
            let restList = concat $ map (\(k, v) -> fmToList' (buf ++ [k]) v) (fmToList m)
            in  case (b) of
                   Nothing -> restList
                   (Just x) -> (buf, x):restList

    lookupFM [] (Trie b m) = b
    lookupFM (k:ks) (Trie b m) = {-# SCC "lookup Trie" #-}
                               do m' <- lookupFM k m
                                  lookupFM ks m'

    bindFM [] x (Trie b m) = Trie (Just x) m
    bindFM (k:ks) x (Trie b m) =
        {-# SCC "bind Trie" #-}
        let t = case (lookupFM k m) of
                    Just t -> t
                    Nothing -> emptyFM
            t' = bindFM ks x t
        in Trie b (bindFM k t' m)

type StringTrie = Trie (AssocList Char) String 
 
