module Test.Center.CommandParser (
    command,
    Command(..),
    Parameter(..)
) where

import Text.ParserCombinators.Parsec
import qualified Text.ParserCombinators.Parsec.Token as P
import Text.ParserCombinators.Parsec.Language

data (Show a) => Command a = Command String [Parameter a]
    deriving Show
data (Show a) => Parameter a = PLiteral String 
                             | PNumber Integer
                             | PCommand (Command a)
                             | PResult a
    deriving Show


-- Wir machen SimpleUnificationProblems zu Instanzen von Read.
instance (Show a) => Read (Command a) where
    readsPrec _ s = parseIt command s

parseIt :: Parser a -> String -> [(a, String)]
parseIt p s = case (parse p "" s) of
                Left err -> []
                Right x -> [(x, "")]

-- Definition des Lexers. Er wird benutzt, um whitespace zu ignorieren.
lexer :: P.TokenParser ()
lexer = P.makeTokenParser haskellDef 

-- Wird definieren Lexikalisch Parser.
whiteSpace = P.whiteSpace lexer
lidentifier = P.identifier lexer
lexeme = P.lexeme lexer
parens = P.parens lexer
stringLiteral = P.stringLiteral lexer
integer = P.integer lexer

command :: (Show a) => Parser (Command a)
command = do whiteSpace
             id <- lidentifier
             l <- parameterList
             return (Command id l)

parameterList :: (Show a) => Parser [Parameter a]
parameterList = do ll <- many parameter 
                   return ll

parameter :: (Show a) => Parser (Parameter a)
parameter = do p <- (innerCommand
                    <|> stringP
                    <|> integerP)
               return p
                
innerCommand :: (Show a) => Parser (Parameter a)
innerCommand = do c <- parens command
                  return (PCommand c)

stringP :: Parser (Parameter a)
stringP = do s <- stringLiteral
             return (PLiteral s)

integerP :: Parser (Parameter a)
integerP = do i <- integer
              return (PNumber i)

run :: Show a => Parser a -> String -> IO ()
run p input = case (parse p "" input) of
                Left err -> do { putStr "parseErr at"
                               ; print err
                               }
                Right x -> print x

{-testTermParse :: String -> IO ()
testTermParse input = run command input-}
