module Main where

import Test.Center.Commands
import Test.Center.IOHelpers

-- individual for command library
data ArgumentType = ArgumentString String
                    | ArgumentInt Int
    deriving Show                    

liftString :: String -> ArgumentType
liftString s = ArgumentString s

liftInt :: Int -> ArgumentType
liftInt i = ArgumentInt i

getCommands :: Commands ArgumentType (IO ())
getCommands = Commands [("help", CommandDescr "Shows Help." [] (showHelp getCommands)),
                        ("info", CommandDescr "Shows info about CCC." [] displayBanner),
                        ("test", CommandDescr "Test Function." 
                            [Argument "StringArg" (liftArgument liftString stringArgument),
                             Argument "Integer" (liftArgument liftInt intArgument)
                            ]
                            
                         testFunction)]

displayBanner :: [ArgumentType] -> IO ()
displayBanner _ = do outputLn "Welcome to ContextMatching Control Center."
                     outputLn "Please enter your commands. Help available via help."
                     outputLn "To exit, press Ctrl-D or enter quit."
                     outputLn ""

testFunction :: [ArgumentType] -> IO ()
testFunction as = do outputLn (show as)

--
-- Main Program
--

process :: String -> IO ()
process cmd = executeCommand cmd getCommands unknown
            
mainLoop :: IO ()
mainLoop = do r <- prompt
              case r of
                (Just s) | s == "quit" -> return ()
                         | length s > 0 -> do process s
                                              mainLoop
                         | otherwise -> mainLoop
                Nothing -> return ()
 


main :: IO ()
main = do displayBanner []
          mainLoop
