module Test.Command (
    Command(..),
    CommandListEntry(..),
    executeCommandEntry
) where

import System.Console.GetOpt

class CommandExecutable cmd opt where
    getCommandName :: cmd opt -> String
    executeCommand :: cmd opt -> [opt] -> IO ()
    getCommandOpts :: cmd opt -> [String] -> String -> IO [opt]

instance CommandExecutable Command opt where
    getCommandName cmd = cmdName cmd
    executeCommand cmd opts =
        let f = cmdFunction cmd
        in  f opts
    getCommandOpts cmd args header =
        let getOptions :: [OptDescr opt] -> [String] -> IO [opt]
            getOptions options argv =
                case (getOpt Permute options argv) of
                    (o,n,[]  ) -> return o
                    (_,_,errs) -> ioError (userError (concat errs ++ usageInfo header options))

        in  getOptions (cmdOptions cmd) args

data Command opt =
    Command {
        cmdName :: String,
        cmdOptions :: [OptDescr opt],
        cmdFunction :: [opt] -> IO ()
    }
data CommandListEntry = forall cmd opt. CommandExecutable cmd opt => CommandListEntry (cmd opt)
    
executeCommandEntry :: [CommandListEntry] 
    -> String   -- Kommando
    -> [String] -- Kommandooptionen
    -> String   -- Usageheader
    -> IO ()
executeCommandEntry [] _ _ header = 
    putStr header
executeCommandEntry ((CommandListEntry cmd):cmds) cmdName cmdOptStrs header
    | (getCommandName cmd == cmdName) = 
            do opts <- getCommandOpts cmd cmdOptStrs header
               executeCommand cmd opts 
    | otherwise = 
            executeCommandEntry cmds cmdName cmdOptStrs header
           
